local hex_grid = require "api.hex_grid"

local gui = {}



function gui.is_frame_opened(player, frame_name)
    return player.gui.relative[frame_name] ~= nil and player.gui.relative[frame_name].visible
end

function gui.hide_frame(player, frame_name)
    local frame = player.gui.relative[frame_name]
    if not frame then return end
    frame.visible = false
end

function gui.show_frame(player, frame_name)
    local frame = player.gui.relative[frame_name]
    if not frame then
        gui.init_hex_core(player)
        frame = player.gui.relative[frame_name]
    end
    frame.visible = true
end

function gui.init_hex_core(player)
    game.print("init hex core")
    local anchor = {gui=defines.relative_gui_type.container_gui, position = defines.relative_gui_position.right}
    local hex_core_gui = player.gui.relative.add {type = "frame", name = "hex-core-gui", direction = "vertical", anchor = anchor}
    hex_core_gui.caption = {"hex-core-gui.title"}
    hex_core_gui.style.size = {width = 280, height = 625}

    local claim_flow = hex_core_gui.add {type = "flow", name = "claim-flow", direction = "vertical"}
    local claim_price = gui.create_coin_tier(claim_flow, "claim-price")
    local claim_hex = claim_flow.add {type = "button", name = "claim-hex", caption = {"hex-core-gui.claim-hex"}, style = "confirm_button"}

    local divider = hex_core_gui.add {type = "line", direction="horizontal"}

    local trades_scroll_pane = hex_core_gui.add {type = "scroll-pane", direction = "vertical"}
    trades_scroll_pane.name = "trades"
end

function gui.update_hex_core(player)
    game.print("update_hex_core")
    local frame = player.gui.relative["hex-core-gui"]
    if not frame then
        gui.init_hex_core(player)
        frame = player.gui.relative["hex-core-gui"]
    end

    local hex_core = player.opened
    if not hex_core then return end

    local state = hex_grid.get_hex_state_from_core(hex_core)
    if not state then return end

    local coin = state.claim_price
    gui.update_coin_tier(frame["claim-flow"]["claim-price"], coin)

    if state.claimed then
        game.print("set invisible")
        -- frame["claim-flow"].visible = false
        frame["claim-flow"]["claim-price"].visible = false
        frame["claim-flow"]["claim-hex"].visible = false
    else
        game.print("set visible")
        -- frame["claim-flow"].visible = true
        frame["claim-flow"]["claim-price"].visible = true
        frame["claim-flow"]["claim-hex"].visible = true
    end
end

function gui.update_coin_tier(flow, coin)
    local hex_coin_sprite = flow['hex-coin']
    local gravity_coin_sprite = flow['gravity-coin']
    local meteor_coin_sprite = flow['meteor-coin']
    local hexaprism_coin_sprite = flow['hexaprism-coin']

    hex_coin_sprite.number = coin.values[1]
    gravity_coin_sprite.number = coin.values[2]
    meteor_coin_sprite.number = coin.values[3]
    hexaprism_coin_sprite.number = coin.values[4]

    local visible = false
    if coin.values[4] > 0 then visible = true end
    hexaprism_coin_sprite.visible = visible

    if coin.values[3] > 0 then visible = true end
    meteor_coin_sprite.visible = visible

    if coin.values[2] > 0 then visible = true end
    gravity_coin_sprite.visible = visible
end

function gui.create_coin_tier(parent, name)
    local flow = parent.add {type = "flow", direction = "horizontal"}
    flow.style.horizontal_spacing = 8
    flow.name = name or "coins"

    local hex_coin_sprite = flow.add {type = "sprite-button", sprite = "hex-coin"}
    hex_coin_sprite.ignored_by_interaction = true
    hex_coin_sprite.name = "hex-coin"
    hex_coin_sprite.style.width = 40
    hex_coin_sprite.style.height = 40
    hex_coin_sprite.number = 1

    local gravity_coin_sprite = flow.add {type = "sprite-button", sprite = "gravity-coin"}
    gravity_coin_sprite.ignored_by_interaction = true
    gravity_coin_sprite.name = "gravity-coin"
    gravity_coin_sprite.style.width = 40
    gravity_coin_sprite.style.height = 40
    gravity_coin_sprite.number = 0

    local meteor_coin_sprite = flow.add {type = "sprite-button", sprite = "meteor-coin"}
    meteor_coin_sprite.ignored_by_interaction = true
    meteor_coin_sprite.name = "meteor-coin"
    meteor_coin_sprite.style.width = 40
    meteor_coin_sprite.style.height = 40
    meteor_coin_sprite.number = 0

    local hexaprism_coin_sprite = flow.add {type = "sprite-button", sprite = "hexaprism-coin"}
    hexaprism_coin_sprite.ignored_by_interaction = true
    hexaprism_coin_sprite.name = "hexaprism-coin"
    hexaprism_coin_sprite.style.width = 40
    hexaprism_coin_sprite.style.height = 40
    hexaprism_coin_sprite.number = 0

    return flow
end

function gui.on_gui_click(event)
    game.print("clicked")
    local player = game.get_player(event.player_index)
    if not player then return end
    game.print(event.element)

    if event.element.name == "claim-hex" then
        game.print("claim hex")
        local hex_core = player.opened
        if not hex_core then
            game.print("[ERROR] Couldn't find hex core")
            return
        end
        local transformation = hex_grid.get_surface_transformation(hex_core.surface)
        local hex_pos = hex_grid.get_hex_containing(hex_core.position, transformation.scale, transformation.rotation)
        hex_grid.claim_hex(player.surface.name, hex_pos)
        gui.update_hex_core(player)
    end
end

return gui
