
local lib = {}



function lib.position_to_string(x, y)
    if not y then
        y = x.y or x[2]
        x = x.x or x[1]
    end
    return "(" .. tostring(x) .. ", " .. tostring(y) .. ")"
end

function lib.vector_add(a, b)
    return {x=(a.x or a[1])+(b.x or b[1]), y=(a.y or a[2])+(b.y or b[2])}
end

function lib.vector_multiply(a, b)
    return {x=(a.x or a[1]) * b, y=(a.y or a[2]) * b}
end

function lib.random_unit_vector(length)
    length = length or 1
    local angle = math.random() * 2 * math.pi
    return {x=length*math.cos(angle), y=length*math.sin(angle)}
end

function lib.startup_setting_value(name)
    local prefixedName = "hextorio-" .. name
    local v = settings.startup[prefixedName]
    if not v then
        lib.log_error("Startup setting " .. name .. " (" .. prefixedName .. ") not found")
        return
    end
    return v.value
end

function lib.table_length(t)
    local count = 0
    for _ in pairs(t) do count = count + 1 end
    return count
end

function lib.table_to_string(t, indent)
    -- courtesy ChatGPT

    indent = indent or 0  -- Keep track of indentation level
    local result = "{\n" -- Start the table representation

    for k, v in pairs(t) do
        local key
        if type(k) == "string" then
            key = string.format("[\"%s\"]", k)
        else
            key = string.format("[%s]", tostring(k))
        end

        local value
        if type(v) == "table" then
            value = lib.table_to_string(v, indent + 2) -- Recursive call for nested tables
        elseif type(v) == "string" then
            value = string.format("\"%s\"", v)
        else
            value = tostring(v)
        end

        result = result .. string.rep(" ", indent + 2) .. key .. " = " .. value .. ",\n"
    end

    result = result .. string.rep(" ", indent) .. "}"
    return result
end

function lib.log_error(txt)
    lib.log(txt, true)
end

function lib.log(txt, error)
    local prefix = "HEXTORIO | "
    if error then
        prefix = prefix .. "\n\nERROR: "
    end
    local s
    if type(txt) == "table" then
        s = prefix .. "\n" .. lib.TableToString(txt)
    else
        s = prefix .. tostring(txt)
    end
    if error then
        s = s .. "\n\n"
    end
    log(s)
end

-- Convert chunk position to rectangular coordinates
function lib.chunk_to_rect(chunk_pos)
    local top_left = {
        x = chunk_pos.x * 32,
        y = chunk_pos.y * 32
    }
    
    local bottom_right = {
        x = top_left.x + 31,
        y = top_left.y + 31
    }
    
    return top_left, bottom_right
end

function lib.is_position_in_rect(position, top_left, bottom_right)
    return position.x >= top_left.x and position.x <= bottom_right.x and position.y >= top_left.y and position.y <= bottom_right.y
end

function lib.unstuck_player(player)
    if not player then return end
    local surface = player.surface
    if not surface then return end
    local position = surface.find_non_colliding_position("character", player.position, 20, 0.5, false)
    if not position then return end
    player.teleport(position, surface)
end

function lib.initial_player_spawn(player)
    if not player then return end
    player.teleport({0, 5}, game.surfaces.nauvis)
    lib.unstuck_player(player)
end



return lib
