
local lib = require "api.lib"
local hex_grid = require "api.hex_grid"
local coin_tiers = require "api.coin_tiers"
local gui = require "api.gui"
local events = require "api.events"

require "commands"



for _, mname in pairs({
    "constants",
    "events",
    "hex_grid",
    "coin_tiers",
}) do
    storage[mname] = require("data." .. mname)
end



script.on_init(function()
    -- nothing to do for now
end)

script.on_event(defines.events.on_tick, function (event)
    if event.tick == 0 then
        events.on_first_frame()
    end
end)

script.on_event(defines.events.on_player_changed_position, function (event)
    local player = game.get_player(event.player_index)
    if not player then return end
    if player.position.x == 0 and player.position.y == 0 then
        if not storage.events.has_game_started then
            events.on_game_started()
        end
    end
end)

script.on_event(defines.events.on_chunk_generated, function(event)
    local surface = event.surface
    local chunk_position = event.position
    hex_grid.on_chunk_generated(surface.name, chunk_position)
end)

script.on_event(defines.events.on_player_main_inventory_changed, function(event)
    local player = game.get_player(event.player_index)
    if not player then return end
    coin_tiers.normalize_inventory(player.get_inventory(defines.inventory.character_main))
end)

script.on_event(defines.events.on_player_trash_inventory_changed, function(event)
    local player = game.get_player(event.player_index)
    if not player then return end
    coin_tiers.normalize_inventory(player.get_inventory(defines.inventory.character_trash))
end)

script.on_event(defines.events.on_player_respawned, function(event)
    -- find open position
    local player = game.get_player(event.player_index)
    if not player then return end
    lib.unstuck_player(player)
end)

script.on_event(defines.events.on_player_created, function(event)
    local player = game.get_player(event.player_index)
    if not player then return end
    lib.initial_player_spawn(player)
end)

script.on_event(defines.events.on_player_joined_game, function(event)
    local player = game.get_player(event.player_index)
    if not player then return end
    lib.unstuck_player(player)
end)

script.on_event(defines.events.on_gui_opened, function (event)
    local player = game.get_player(event.player_index)
    if not player then return end

    if event.entity then
        if event.entity.name == "hex-core" then
            gui.show_frame(player, "hex-core-gui")
            gui.update_hex_core(player)
        end
    end
end)

script.on_event(defines.events.on_gui_click, function (event)
    gui.on_gui_click(event)
end)
